# Standard Imports
import urllib.request
import json


class Player:
    """Class with attributes of a Runescape Player using Jagex's RS3 API

    Most of Runescape 3's API can be accessed at: http://runescape.wikia.com/wiki/Application_programming_interface

    ...

    Parameters:
    name : str
        Name of the Player, gets set to case-sensitive name later if his RuneMetrics profile is
        not private. If you're also working with clans, the player's RuneMetrics profile is private, his name should be
        passed case-sensitively. Otherwise some clan attributes will not be available.

    Attributes:
    ----------
    name : str
        Name of the Player, gets set to case-sensitive name later if his RuneMetrics profile is not private.
    exp : int or None
        Total Exp of the player, or None if private_profile is True.
    combat_level : int or None
        Total Combat Level of the player, or None if private_profile is True.
    total_level : int or None
        Total Skill Level of the player, or None if private_profile is True.
    private_profile : bool
        If the player has a private RuneMetrics profile or not.
    exists : bool
        If the player exists or not.
    """

    def __init__(self, name: str):

        self.name = name
        self.exp = None
        self.combat_level = None
        self.total_level = None
        self.private_profile = True
        self.exists = True

        # If user's runemetrics profile is private, self.name will be the same as passed when creating object.
        # Otherwise it will get the correct case-sensitive name from his runemetrics profile.
        # Some other info like Total exp, Combat level and Total level will be created as well.
        if self.runemetrics_info():
            self.metrics_info = self.runemetrics_info()
            self.name = self.metrics_info['name']
            self.exp = self.metrics_info['totalxp']
            self.combat_level = self.metrics_info['combatlevel']
            self.total_level = self.metrics_info['totalskill']
        self.info = self._dict_info()
        self.suffix = self.info['is_suffix']
        self.title = self.info['title']
        try:
            self.clan = self.info['clan']
        except KeyError:
            self.clan = None

    def _raw_info(self):
        """
        Gets JsonP information on a player, in string format.

        Returns
        ------
        str
            JsonP information on player, converted to string.

        ------
        .. note:: Has to be formatted correctly into Json or other formats to be worked with properly first.
        """

        info_url = (f"http://services.runescape.com/m=website-data/playerDetails.ws?names=%5B%22{self.name}"
                    f"%22%5D&callback=jQuery000000000000000_0000000000&_=0")
        client = urllib.request.urlopen(info_url)
        return str(client.read())

    def _dict_info(self):
        """
        Gets the raw string info from self._raw_info() and formats it into Dictionary format.:
        Used to set self.info.

        Returns
        ------
        dict
            Dictionary is formatted as follows::
                >>> player_info = {
                ... 'is_suffix': True,
                ... 'recruiting': True,
                ... 'name': 'nriver',
                ... 'clan': 'Atlantis',
                ... 'title': 'the Liberator'
                ... }
            is_suffix : bool
                If the player's title is a Suffix or not
            recruiting : bool
                If the player's clan is set as Recruiting or not
            name : str
                The player's name, passed as is when creating object Player
            clan : str
                The player's clan name
            title : str
                The player's current title
        """
        str_info = self._raw_info()
        info_list = []
        # str_info[36] = Start of json format in URL '{'
        for letter in str_info[36:]:
            info_list.append(letter)
            if letter == '}':
                break
        info_list = ''.join(info_list)
        info_dict = json.loads(info_list)
        info_dict['is_suffix'] = info_dict['isSuffix']
        del info_dict['isSuffix']
        return info_dict

    def runemetrics_info(self):
        """
        Gets player info from Runemetrics' API.
        Sets self.private_profile to True if his Runemetrics profile is found to be Private.

        Returns
        ------
        dict or bool
            Runemetrics player info if the player profile is public, or False if it is Private.
        """
        user_name = self.name.replace(' ', '%20')
        info_url = f"https://apps.runescape.com/runemetrics/profile/profile?user={user_name}&activities=0"
        client = urllib.request.urlopen(info_url)
        info = client.read()
        json_info = json.loads(info)
        try:
            if json_info['error'] == 'PROFILE_PRIVATE':
                self.private_profile = True
                return False
            if json_info['error'] == 'NO_PROFILE':
                self.private_profile = False
                self.exists = False
                return False
        except KeyError:
            self.private_profile = False
            return json_info


if __name__ == '__main__':
    # Creating Player with the name "nriver"
    player = Player("nriver")

    # Player name (Actual case-sensitive name if Runemetrics profile isn't private, otherwise it will be as passed)
    print(player.name)

    # Player info
    print(player.info)

    # Player clan
    print(player.clan)

    # Player title
    print(player.title)

    # If the player's title is a suffix or not
    print(player.suffix)

    # Prints the total player exp (if his Runemetrics profile is private it will output 0)
    print(player.exp)

    # Prints the Combat level of player (if his Runemetrics profile is private it will output 0)
    print(player.combat_level)

    # Prints the Total level of player (if his Runemetrics profile is private it will output 0)
    print(player.total_level)



import setuptools

__author__ = None
__version__ = None

# Reading long description from README.md
with open("README.md", "r") as fh:
    long_description = fh.read()

# Reading __version__ and __author__ from __init__.py
with open("rs3clans/__init__.py") as f:
    exec(f.read())

AUTHOR = __author__
VERSION = __version__
NAME = 'rs3clans'
EMAIL = 'johnvictorfs@gmail.com'
DESCRIPTION = 'A Python 3 module wrapper for RuneScape 3 Clan\'s API'
URL = 'https://github.com/johnvictorfs/rs3clans.py'
REQUIRED = [
    'requests>=2.19.1',
]
REQUIRES_PYTHON = '>=3.6.0'

setuptools.setup(
    name=NAME,
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type='text/markdown',
    setup_requires=REQUIRED,
    python_requires=REQUIRES_PYTHON,
    version=VERSION,
    author=AUTHOR,
    author_email=EMAIL,
    license='MIT',
    packages=setuptools.find_packages(exclude=('tests',)),
    zip_safe=False,
    url=URL,
    classifiers=(
        "Programming Language :: Python :: 3.6"
    )
)



# Standard Imports
import csv
import requests


class ClanNotFoundError(Exception):
    """
    Error exception to be called when a clan is not found when trying to read the clan's URL using Rs3's Official API.

    Correct error handling when reading a clan::
        >>> import rs3clans
        >>> try:
        ...     clan_name = "oasdiuahsiubasiufbasuf"
        ...     clan = rs3clans.Clan(name=clan_name)
        ... except rs3clans.ClanNotFoundError:
        ...     print("Exception encountered!")
        ...
        'Exception encountered!'
    """

    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


class Clan:
    """Class with attributes of a Runescape Clan using Jagex's RS3 API

    Most of Runescape 3's API can be accessed at: http://runescape.wikia.com/wiki/Application_programming_interface

    ...

    Parameters
    ----------
    name : str
        The name of the clan. Does not need to be case-sensitive.
    set_exp : bool, optional
        If True, set.exp will be set with the total Exp from the clan. False by default.
    set_dict : bool, optional
        If True, self.member will be set with info from all members from the clan in dict format. True by default.

    Attributes
    ----------
    name : str
        The name of the clan. Set when creating object.
    exp : int or None
        The total Exp of the clan, or None if argument set_exp is False (False by default).
    member : dict or None
        All info from members in the clan in dictionary format as below::
            >>> member_info = {
            ...     'player_1': {
            ...         'exp': 225231234,
            ...         'rank': 'Leader'
            ...     },
            ...     'player_2': {
            ...     'exp': 293123082,
            ...     'rank': 'Overseer'
            ...    },
            ... }
        or None if argument set_dict is False (True by default).
    count : int
        The number of members in the Clan.
    avg_exp : int or None
         The average clan exp per member in the Clan, or None if either set_exp or set_dict arguments are False.

    Raises
    ------
    ClanNotFoundError
        If an invalid clan name is passed when creating object Clan.
    """

    def __init__(self, name, set_exp=False, set_dict=True):
        self.name = name
        self.exp = None
        self.member = None

        if set_exp is True:
            self.exp = self._list_sum()
        if set_dict is True:
            self.member = self.dict_lookup()
            self.count = len(self.member)
        if set_exp is True and set_dict is True:
            self.avg_exp = self.exp / self.count

    def list_lookup(self):
        """

        Used for getting all information available from a clan using Rs3's Clan API.
        Mainly used for calculating the total exp of a clan manually.

        Returns
        --------
        list
            All players information from a clan in list format like so::
                >>> clan_list = [
                ... ['Clanmate', ' Clan Rank', ' Total XP', ' Kills'],
                ... ['Pedim', 'Owner', '739711654', '2'],
                ... ['Tusoroxo', 'Deputy Owner', '1132958333', '0'],
                ... ]
        """
        clan_url = f'http://services.runescape.com/m=clan-hiscores/members_lite.ws?clanName={self.name}'

        with requests.Session() as session:
            download = session.get(clan_url)
            decoded = download.content.decode('windows-1252')
            clan_list = list(csv.reader(decoded.splitlines(), delimiter=','))
            if clan_list[0][0] != "Clanmate":
                raise ClanNotFoundError(f"Couldn't find clan: {self.name}")
            for row in clan_list:
                row[0] = row[0].replace(r"\xa0", " ")
            return clan_list

    def dict_lookup(self, rank_key="rank", exp_key="exp"):
        """

        Used for getting all information available from a clan using Rs3's Clan API.

        Contrary to `list_lookup` it returns it as a Dictionary format instead.
        The dict format makes easier to find info specific to certain members of the Clan instead of looping over it.

        It's used for setting :py:attr: `member` when set_dict argument is passed as True when creating a Clan object.

        Parameters
        --------
        rank_key : str
            The key for Clan Rank to be used in the returned dictionary with all members info of clan.
        exp_key : str
            The key for Clan Exp to be used in the returned dictionary with all members info of clan.

        Returns
        --------
        dict
            Information from all members in the clan in dictionary format for ease of access.
        """
        clan_list = self.list_lookup()
        clan_dict = {}
        for row in clan_list[1:]:
            user_rank = row[1]
            username = row[0]
            user_exp = int(row[2])

            clan_dict[username] = {
                rank_key: user_rank,
                exp_key: user_exp
            }
        return clan_dict

    def _list_sum(self):
        """
        .. highlight:: numpy

        Gets a clan list from :func: `list_lookup` and sums the total exp of the clan.
        It's used for setting :py:attr: `exp` when set_exp argument is passed as True when creating a Clan object.

        Returns
        --------
        int
            The total Exp of the Clan.
        """
        clan_list = self.list_lookup()
        return sum(int(row[2]) for row in clan_list[1:])

    def _dict_sum(self):
        """
        .. highlight:: numpy

        Gets a clan dictionary from :func: `dict_lookup` and sums the total exp of the clan.

        .. deprecated:: 0.4.0
            Use :func: `list_sum` as it's much faster.

        Returns
        --------
        int
            The total Exp of the Clan.
        """
        clan_dict = self.dict_lookup()
        return sum(members['exp'] for members in clan_dict.values())


if __name__ == '__main__':
    # Create Clan with the name "Atlantis"
    clan = Clan(name="Atlantis", set_exp=True)

    # Info from member "NRiver" (case-sensitive)
    print(clan.member['NRiver'])

    # Total Exp of the clan
    print(clan.exp)

    # Average Clan Exp per Member of the clan
    print(clan.avg_exp)

    # Clan Name as passed when creating object Clan
    print(clan.name)

    # Number of members in clan
    print(clan.count)

    clan_two = Clan("okafbjhvfjsdhbsdhfhbqjfbqwf")
    print(clan.name)
